<?php
/*
 *  $Id$
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 * This software consists of voluntary contributions made by many individuals
 * and is licensed under the MIT license. For more information, see
 * <http://www.doctrine-project.org>.
 */

namespace Doctrine\DBAL;

/**
 * Container for all DBAL events.
 *
 * This class cannot be instantiated.
 *
 * @author Roman Borschel <roman@code-factory.org>
 * @since 2.0
 */
final class Events
{
    private function __construct() {}

    const postConnect = 'postConnect';

    const onSchemaCreateTable             = 'onSchemaCreateTable';
    const onSchemaCreateTableColumn       = 'onSchemaCreateTableColumn';
    const onSchemaDropTable               = 'onSchemaDropTable';
    const onSchemaAlterTable              = 'onSchemaAlterTable';
    const onSchemaAlterTableAddColumn     = 'onSchemaAlterTableAddColumn';
    const onSchemaAlterTableRemoveColumn  = 'onSchemaAlterTableRemoveColumn';
    const onSchemaAlterTableChangeColumn  = 'onSchemaAlterTableChangeColumn';
    const onSchemaAlterTableRenameColumn  = 'onSchemaAlterTableRenameColumn';
    const onSchemaColumnDefinition        = 'onSchemaColumnDefinition';
    const onSchemaIndexDefinition         = 'onSchemaIndexDefinition';
}
